--------------------------------------------------------------------------------
-- Module Declaration
--

local mod = BigWigs:NewBoss("Ji-Kun Assist", 930)
if not mod then return end
mod:RegisterEnableMob(69712)

local mainMod = BigWigs:GetBossModule("Ji-Kun")

local ML = mainMod:GetLocale("enUS", true)

local nestCounter = 0

local L = mod:NewLocale("enUS", true)
if L then
	L.assist = "Ji-Kun Assist"
	L.assist_desc = "Turn Ji-Kun Assist on or off"
	L.your_nest_soon = "Your nest in 5 sec!"
end
L = mod:GetLocale()

function mod:GetOptions()
	return {
		"assist",
	}, {
		["assist"] = "general",
	}
end

function mod:OnBossEnable()
	self:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT", "CheckBossStatus")
	self:RegisterEvent("CHAT_MSG_MONSTER_EMOTE")
end

function mod:OnEngage()
	nestCounter = 0
end

--------------------------------------------------------------------------------
-- Event Handlers
--

local jikun_assignment = {
	[4] = {
1,
2,
3,
4,
5,
1,
2,
3,
1,
4,
2,
5,
3,
4,
5,
1,
2,
1,
3,
2,
1,
3,
2,
4,
5,
3,
1,
4,
	},
	[6] = {
1,
2,
3,
4,
1,
5,
2,
3,
1,
4,
2,
5,
3,
1,
4,
2,
5,
1,
3,
2,
4,
1,
3,
5,
2,
4,
1,
3,
5,
2,
4,
5,
3,
1,
4,
2,
5,
	},
}

do
	local function preWarnNest()
		mod:Message("assist", "Personal", "Long", L["your_nest_soon"], false)
	end
	local function stopMainModBars(nextNest)
		mainMod:StopBar(("(%d) %s"):format(nextNest, ML["lower_nest"]))
		mainMod:StopBar(("(%d)%s+(%d)%s"):format(nextNest, ML["down"], nextNest+1, ML["up"]))
		mainMod:StopBar(("(%d)%s+(%d)%s"):format(nextNest, ML["up"], nextNest+1, ML["down"]))
		mainMod:StopBar(("(%d) %s"):format(nextNest, ML["upper_nest"]))
		mainMod:StopBar(("(%d) %s (%s)"):format(nextNest, ML["down"], ML["add"]))
		mainMod:StopBar(("(%d)%s(%s)+(%d)%s"):format(nextNest, ML["down"], ML["add"], nextNest+1, ML["up"]))
		mainMod:StopBar(("(%d)%s+(%d)%s(%s)"):format(nextNest, ML["down"], nextNest+1, ML["up"], ML["add"]))
		mainMod:StopBar(("(%d)%s+(%d)%s(%s)+(%d)%s"):format(nextNest, ML["up"], nextNest+1, ML["down"], ML["add"], nextNest+2, ML["up"]))
		mainMod:StopBar(("(%d)%s+(%d)%s+%d(%s)"):format(nextNest, ML["down"], nextNest+1, ML["up"], nextNest+2, ML["down"]))
		mainMod:StopBar(("(%d)%s+(%d)%s+%d(%s)"):format(nextNest, ML["up"], nextNest+1, ML["down"], nextNest+2, ML["up"]))
	end
	function mod:CHAT_MSG_MONSTER_EMOTE(_, msg)
		if msg:find(ML["upper_hatch_trigger"]) or msg:find(ML["lower_hatch_trigger"]) then
			local myGroup = 0
			for i=1, GetNumGroupMembers() do
				if UnitName("player") == GetRaidRosterInfo(i) then
					myGroup = select(3, GetRaidRosterInfo(i))
				end
			end
			local diff = self:Difficulty()
			nestCounter = nestCounter + 1
			local nextNest = nestCounter + 1
			local text = (msg:find(ML["upper_hatch_trigger"])) and ("GO %s NOW!"):format(ML["upper_nest"]) or ("GO %s NOW!"):format(ML["lower_nest"])
			local icon = (msg:find(ML["upper_hatch_trigger"])) and "misc_arrowlup" or "misc_arrowdown"
			if diff == 4 then -- 25 N
				if not jikun_assignment[4][nestCounter+1] then print("Need more data!") return end
				if myGroup == jikun_assignment[4][nestCounter] then
					self:Message("assist", "Personal", "Long", text, icon)
				elseif jikun_assignment[4][nestCounter+1] and myGroup == jikun_assignment[4][nestCounter+1] then
					self:ScheduleTimer(preWarnNest, 25)
				end
				if nestCounter % 28 < 4 or nestCounter % 28 == 12 or nestCounter % 28 == 13 or nestCounter % 28 == 23 then
					self:Bar("assist", 30, ML["lower_nest"]..(" {%d}"):format(jikun_assignment[4][nestCounter+1]), "misc_arrowdown")
				elseif nestCounter % 28 == 4 or nestCounter % 28 == 8 or nestCounter % 28 == 10 or nestCounter % 28 == 14 or nestCounter % 28 == 17 or nestCounter % 28 == 19 or nestCounter % 28 == 21 or nestCounter % 28 == 24 or nestCounter % 28 == 26 then -- up and down at same time
					if not jikun_assignment[4][nestCounter+2] then return end
					self:Bar("assist", 30, ("%s{%d}+%s{%d}"):format(ML["down"], jikun_assignment[4][nestCounter+1], ML["up"], jikun_assignment[4][nestCounter+2]), 134347)
					if myGroup == jikun_assignment[4][nestCounter+2] then
						self:ScheduleTimer(preWarnNest, 25)
					end
				elseif nestCounter % 28 == 6 or nestCounter % 28 == 7 or nestCounter % 28 == 16 then
					self:Bar("assist", 30, ML["upper_nest"]..(" {%d}"):format(jikun_assignment[4][nestCounter+1]), "misc_arrowlup")
				end
			elseif diff == 6 then -- 25 H
				if not jikun_assignment[6][nestCounter+1] then print("Need more data!") return end
				if myGroup == jikun_assignment[6][nestCounter] then
					self:Message("assist", "Personal", "Long", text, icon)
				end
				if nestCounter < 3 or nestCounter == 7 or nestCounter == 12 then
					if myGroup == jikun_assignment[6][nestCounter+1] then
						self:ScheduleTimer(preWarnNest, 25)
					end
				elseif nestCounter == 3 or nestCounter == 5 or nestCounter == 8 or nestCounter == 10 or nestCounter == 13 or nestCounter == 15 or nestCounter == 17 or nestCounter == 19 or nestCounter == 24 or nestCounter == 29 then
					if myGroup == jikun_assignment[6][nestCounter+1] or myGroup == jikun_assignment[6][nestCounter+2] then
						self:ScheduleTimer(preWarnNest, 25)
					end
				elseif nestCounter == 21 or nestCounter == 26 or nestCounter == 31 or nestCounter == 34 then
					if myGroup == jikun_assignment[6][nestCounter+1] or myGroup == jikun_assignment[6][nestCounter+2] or myGroup == jikun_assignment[6][nestCounter+3] then
						self:ScheduleTimer(preWarnNest, 25)
					end
				end
				if nestCounter < 3 or nestCounter == 12 then
					self:Bar("assist", 30, ("(%d) %s"):format(jikun_assignment[6][nextNest], ML["lower_nest"]), "misc_arrowdown")
				elseif nestCounter == 3 or nestCounter == 8 or nestCounter == 17 or nestCounter == 19 or nestCounter == 29 then
					self:Bar("assist", 30, ("(%d)%s+(%d)%s"):format(jikun_assignment[6][nextNest], ML["down"], jikun_assignment[6][nextNest+1], ML["up"]), 134347)
				elseif nestCounter == 13 or nestCounter == 15 or nestCounter == 24 then
					self:Bar("assist", 30, ("(%d)%s+(%d)%s"):format(jikun_assignment[6][nextNest], ML["up"], jikun_assignment[6][nextNest+1], ML["down"]), 134347) -- this is intentional, because this is how blizzard announces it too!
				elseif nestCounter == 7 then
					self:Bar("assist", 30, ("(%d) %s"):format(jikun_assignment[6][nextNest], ML["upper_nest"]), "misc_arrowlup")
				elseif nestCounter == 1 then
					self:Bar("assist", 30, ("(%d) %s (%s)"):format(jikun_assignment[6][nextNest], ML["down"], ML["add"]), "misc_arrowdown")
				elseif nestCounter == 5 then
					self:Bar("assist", 30, ("(%d)%s(%s)+(%d)%s"):format(jikun_assignment[6][nextNest], ML["down"], ML["add"], jikun_assignment[6][nextNest+1], ML["up"]), 134347)
				elseif nestCounter == 10 then
					self:Bar("assist", 30, ("(%d)%s+(%d)%s(%s)"):format(jikun_assignment[6][nextNest], ML["down"], jikun_assignment[6][nextNest+1], ML["up"], ML["add"]), 134347)
				elseif nestCounter == 21 then
					self:Bar("assist", 30, ("(%d)%s+(%d)%s(%s)+(%d)%s"):format(jikun_assignment[6][nextNest], ML["up"], jikun_assignment[6][nextNest+1], ML["down"], ML["add"], jikun_assignment[6][nextNest+2], ML["up"]), 134347) -- XXX keep an eye out for this if it fits on the bar
				elseif nestCounter == 26 or nestCounter == 34 then
					self:Bar("assist", 30, ("(%d)%s+(%d)%s+%d(%s)"):format(jikun_assignment[6][nextNest], ML["down"], jikun_assignment[6][nextNest+1], ML["up"], jikun_assignment[6][nextNest+2], ML["down"]), 134347)
				elseif nestCounter == 31 then
					self:Bar("assist", 30, ("(%d)%s+(%d)%s+%d(%s)"):format(jikun_assignment[6][nextNest], ML["up"], jikun_assignment[6][nextNest+1], ML["down"], jikun_assignment[6][nextNest+2], ML["up"]), 134347)
				end
			end
			self:ScheduleTimer(stopMainModBars, 0.1, nextNest)
		end
	end
end

